@ECHO OFF
COLOR 02

REM ============================================================================
REM this batch file attemtps to backup your files using 7za.exe -- command line
REM version on 7Zip; it can be downloaded at http://www.7-zip.org/download.html
REM ----------------------------------------------------------------------------
REM #backup directory will be created, if it doesn't exist
REM
REM if you set "full" mode:
REM - all files from all subdirectories (with the exception of #backup) will be
REM   archived, unless you provide a file with filenames (or extensions, in the
REM   form of "*.ext"), one per line, to archive; even then files without
REM   extensions will be archived, though
REM   - default name for this file is "full.txt"
REM
REM if you set "partial" mode:
REM - only files set explicitly in "partial.tmp" file (default name; you can
REM   provide other) will be archived
REM ============================================================================

IF NOT EXIST "C:\Program Files\7za\7za.exe" GOTO archiver_doesnt_exist

IF "%1" == "" GOTO no_project_name_given
IF NOT EXIST %1 GOTO project_doesnt_exist
IF "%2" == "" GOTO no_mode_given
SET archive_name=%date:/=%@%time:~0,2%h%time:~3,2%m
REM name will be e.g. full.2014-07-15@11h11m.7z or partial.2014-07-15@11h11m.7z
REM "full" or "partial" prefix are added below in the code
IF "%2" == "-f" GOTO full
IF "%2" == "-p" GOTO partial
GOTO bad_mode_given

:archiver_doesnt_exist
ECHO *** Error: 7za.exe hasn't been found
GOTO end

:no_project_name_given
ECHO *** Error: you must pass project name and mode
ECHO:
ECHO Usage:
ECHO:
ECHO   backup.bat ^<project_name^> ^<mode^> [file_list]
ECHO:
ECHO   project_name : directory, in which your files are kept
ECHO   mode         : either -f (full) or -p (partial)
ECHO   file_list    : name of file in #backup subdirectory, containig list of
ECHO                  extensions (full mode) or files (partial mode) to backup
ECHO                  -- this is optional: if none is given, "full.txt" is
ECHO                     assumed for full and "partial.tmp" for partial backup
GOTO end

:project_doesnt_exist
ECHO *** Error: project directory does not exist
GOTO end

:no_mode_given
ECHO *** Error: you must pass backup mode (-f for full or -p for partial)
GOTO end

:bad_mode_given
ECHO *** Error: unsupported mode; you may use only -f for full or -p for partial backup
GOTO end

:full
CD %1
IF "%3" == "" SET list=full.txt
IF NOT "%3" == "" SET list=%3
IF NOT EXIST #backup\%list% GOTO full_no_filelist
REM archive file with no extensions:
"C:\Program Files\7za\7za.exe" a #backup\full.%archive_name%.7z -r * -x!*.* -x!#arch\ -x!#backup\ -w#backup
REM add files with extensions (from the %list%):
"C:\Program Files\7za\7za.exe" a #backup\full.%archive_name%.7z @#backup\%list% -r -x!#arch\ -x!#backup\ -w#backup
CD ..
GOTO end

:full_no_filelist
REM archive all files in project's directory (except of \#backup)
"C:\Program Files\7za\7za.exe" a #backup\full.%archive_name%.7z * -r -x!#arch\ -x!#backup\ -w#backup
CD ..
GOTO end

:partial
CD %1
IF "%3" == "" SET list=partial.tmp
IF NOT "%3" == "" SET list=%3
IF NOT EXIST #backup\%list% GOTO partial_no_filelist
"C:\Program Files\7za\7za.exe" a #backup\partial.%archive_name%.7z @#backup\%list% -r -x!#arch\ -x!#backup\ -w#backup
CD ..
GOTO end

:partial_no_filelist
ECHO *** Error: list of files (%list%) not found in %1%\#backup directory
CD ..
GOTO end

:end