/****
    *   Simple Project Files Backup System for RJ TextEdit
    *   rev. 1, 2015/01/28
    *   ------------------------------------------------------------------------
    *   THE BEERWARE LICENSE
    *   PJJ wrote this script. As long as you retain this notice, you can do
    *   with it ("it" meaning both this script and its companion "backup.bat"
    *   file) whatever you want. If we meet some day, and you think this stuff
    *   is worth it, you can buy me a beer in return.
    *
    *   http://www.tldrlegal.com/license/beerware-license
    *   ------------------------------------------------------------------------
    */

// ---- tweak this to your needs: ----------------------------------------------
var rootDir = "D:\\wwwJobs\\";  // directory in which project subdirectories
                                // are located; should end with trailing slash!

// ---- you don't really need to change values below ---------------------------
var commentFilename = "!README";// name of file with backup comment

var showOutput = true;         // whether to show output in RJ TE bottom bar;
                                // good for debugging

var openFolder = false;         // whether to open #backup folder after 7za has run
// ---- no more tweaking needed below this line --------------------------------

// define variables for dialogs
var d;                          // main form window
var l, p, f, m, e;              // backup mode choice dialog
var ch, c, ok;                  // comment dialog
var partialList;                // list of opened files, will be saved as
                                // #backup/partial.tmp and then deleted
var howManyOpenDocuments;
var currentDocName;

Document.BeginUpdate();

// find project name
s = ScriptUtils.StringReplaceAll(Document.FileName, rootDir, "-");
i = ScriptUtils.Pos("\\", s);
projectName = copy(s, 2, i - 2);
if ((projectName == "") || (projectName == ":")) {
    projectName = "undefined";
    ShowMessage("Current file is outside the project root (i.e. " + rootDir + ")\nNothing to do, quitting.");
}

function createPartialList()
{
    // create list of currently open files that belong to a project
    // and save it into #backup/partial.tmp
    currentDocName = Document.FileName;
    // remove rootDir and projectName from currentDocName, and then a slash
    currentDocName = copy(currentDocName, length(rootDir) + length(projectName) + 2, length(currentDocName));
    partialList = currentDocName;

    // if there are more open documents, add them up to fileList
    howManyOpenDocuments = MainApp.DocumentCount;
    if (howManyOpenDocuments > 1) {
        MainApp.NextDoc();
        for (i = 1; i < howManyOpenDocuments; i++) {
            currentDocName = Document.FileName;
            // check if this document belongs to the current project
            if (pos(rootDir + projectName, currentDocName) == 1) {
                currentDocName = copy(currentDocName, length(rootDir) + length(projectName) + 2, length(currentDocName));
                partialList = partialList + "\n" + currentDocName;
            }
            MainApp.NextDoc();
        }
    }
    // add readme file, if it was created
    if (c.lines.Text != "") {
        partialList = partialList + "\n" + commentFilename;
    }

    // save partialList as #backup/partial.tmp
    WStrings.SetText(partialList);
    WStrings.SaveToFile(rootDir + projectName + "/#backup/partial.tmp");
}

function readComment(Sender)
{
    // hide elements of backup mode choice dialog; don't hide window d, though!
    l.Hide;
    p.Hide;
    f.Hide;
    m.Hide;
    e.Hide;
    // show elements of comment dialog:
    ch.Show;
    c.Show; c.SetFocus;
    ok.Show;
    ok.Name = Sender.Name + "1";
}

function saveComment(Sender)
{
    // hide the form
    d.Hide;
    // create #backup subdirectory
    MainApp.RunAppAndWait("c:\\windows\\system32\\cmd.exe", "/c (md " + rootDir + projectName + "\\#backup)", false, false);
    // call backup function
    if (Sender.Name == "partial1") {
        doBackup(Sender);
    }
    else if (Sender.Name == "full1") {
        doBackup(Sender);
    }
    else {
        ShowMessage("Unrecognized backup mode. Quitting...");
        exit();
    }
}

function logBackup(mode)
{
    var entries = "";
    var newEntry = "";
    var backupModeDesc;
    if (mode == "partial1") {
        backupModeDesc = "partial    ";
    }
    else {
        backupModeDesc = "full       ";
    }
    newEntry = backupModeDesc + DateToStr(Date()) + " " + copy(TimeToStr(Time()), 0, 5);
    if (c.lines.Text != "") {
        var comment = ScriptUtils.StringReplaceAll(c.lines.Text, "\r\n", "; ");
        newEntry = newEntry + "    " + comment;
    }
    var filename = rootDir + projectName + "\\#backup\\backup.log";
    var size = FileSize(filename);

    if (FileExists(filename)) {
        Word mode = fmOpenReadWrite;
        TFileStream stream = TFileStream.Create(filename, mode);
        try {
            entries = stream.ReadString(size, encAnsi);
            entries = entries + "\n" + newEntry;
        }
        finally {
             stream.Free;
        }
    }
    else {
        // backup.log doesn't exist yet
        Word mode = fmCreate or fmOpenReadWrite;
        entries = "mode       date & time         comment" + "\n" +
        "=========================================" + "\n" + newEntry;
    }

    // write to the log file
    TFileStream stream = TFileStream.Create(filename, mode);
    try {
        stream.WriteString(entries, encAnsi);
    }
    finally {
        stream.Free;
    }
}

function doBackup(Sender)
{
    // write down comment (if not empty)
    if (c.lines.Text != "") {
        WStrings.SetText(c.lines.Text);
        WStrings.SaveToFile(rootDir + projectName + "\\" + commentFilename);
    }
    if (Sender.Name == "partial1") {
        createPartialList();
        MainApp.RunAppAndWait(rootDir + "\\backup.bat", projectName + " -p partial.tmp", showOutput, true);
        ShowMessage("Partial backup was done and saved in " + rootDir + projectName + "\\#backup directory");
        // delete partial.tmp file
        MainApp.RunAppAndWait("c:\\windows\\system32\\cmd.exe", "/c (del " + rootDir + projectName + "\\#backup\\partial.tmp)", false, false);
        // delete readme file, if it was created
        if (c.lines.Text != "") {
            MainApp.RunAppAndWait("c:\\windows\\system32\\cmd.exe", "/c (del " + rootDir + projectName + "\\" + commentFilename + ")", false, false);
        }
        if (openFolder) {
            MainApp.RunAppAndWait("c:\\windows\\explorer.exe", rootDir + projectName + "\\#backup", showOutput, false);
        }
    }
    else if (Sender.Name == "full1") {
        MainApp.RunAppAndWait(rootDir + "\\backup.bat", projectName + " -f full.txt", showOutput, true);
        ShowMessage("Full backup was done and saved in " + rootDir + projectName + "\\#backup directory");
        // delete readme file, if it was created
        if (c.lines.Text != "") {
            MainApp.RunAppAndWait("c:\\windows\\system32\\cmd.exe", "/c (del " + rootDir + projectName + "\\" + commentFilename + ")", false, false);
        }
        if (openFolder) {
            MainApp.RunAppAndWait("c:\\windows\\explorer.exe", rootDir + projectName + "\\#backup", showOutput, false);
        }
    }
    logBackup(Sender.Name);
    exit();
}

function exit()
{
    d.ModalResult = mrOk;
}

// ------------------------------------------------------------ main form window
d = new TForm(nil);
d.Caption = "Simple Project Files Backup System";
d.BorderStyle = bsDialog;
d.Position = poScreenCenter;
d.Width = 350;
d.Height = 185;
var w = d.ClientWidth;
var h = d.ClientHeight;

// ---- choice of backup mode
l = new TListBox(d);
l.Parent = d;
l.SetBounds(1, 1, w - 1, h - 1); // x, y, w, h
l.Items.Text = "\n   project name: " + projectName;

p = new TButton(d);
p.Name = "partial";
p.Parent = d;
p.SetBounds(10, 40, 155, 35);
p.Caption = "partial (= open documents)";
p.OnClick = &readComment;

if (projectName != "undefined") {
    f = new TButton(d);
    f.Name = "full";
    f.Parent = d;
    f.SetBounds(180, 40, 155, 35);
    f.Caption = "full (= project directory)";
    f.OnClick = &readComment;
}

m = new TListBox(d);
m.Parent = d;
m.SetBounds(10, 90, 155, 55);
m.Items.Text = "\n      Save early, save often\n                         Al Lowe";

e = new TButton(d);
e.Parent = d;
e.SetBounds(w - 90, h - 35, 75, 25);
e.Anchors = akRight + akBottom;
e.Caption = "cancel";
e.OnClick = &exit;

// ---- comment dialog

// hint
ch = new TLabel(d);
ch.Parent = d;
ch.SetBounds(20, 10, 80, 20); // x, y, w, h
ch.Caption = "Enter comment for your backup (it will be saved in a filed named\n" + commentFilename
    + ") or just click on the button below to skip it.";
ch.Hide;

// input box
c = new TMemo(d);
c.Parent = d;
c.Hint = "comment your backup here";
c.ShowHint = true;
c.SetBounds(20, 50, 305, 60);
c.Hide;

// OK button
ok = new TButton(d);
ok.Parent = d;
ok.SetBounds(130, 125, 95, 25);
ok.Caption = "OK";
ok.OnClick = &saveComment;
ok.Hide;

// show the main dialog
if (projectName != "undefined") {
    d.ShowModal;
    d.Free;
}